/* pipeprg.cc
* This file is part of Colorifer
* Copyright (c) 2002,2003,2004 by Stanislav Ievlev
*
* This file is covered by the GNU General Public License,
* which should be included with colorifer as the file COPYING.
*/

#include <stdexcept>

#include <unistd.h>
#include <sys/wait.h>

#include <pipeprg.hh>

using namespace alt;
using namespace colorifer;


void colorifer::run(const std::string& execname, char *args[])
{
	::execv(execname.c_str(), args);
	throw std::runtime_error("unable to execv " + execname);
}

pipe_prg::pipe_prg(const std::string& execname, char *args[]):
		child_rc_(0),
		child_pid_( -1)
{
	fds_stdout_[0] = -1;
	fds_stdout_[1] = -1;

	fds_stderr_[0] = -1;
	fds_stderr_[1] = -1;

	GLIBC_ASSERT(::pipe(fds_stdout_));
	GLIBC_ASSERT(::pipe(fds_stderr_));

	//fork subprocess
	GLIBC_ASSERT(child_pid_ = ::fork());

	if (!child_pid_)
		child(execname, args); //run subprocess ...
	else //... and return to main program. Now we have only pipe with subprocess
	{
		::close(fds_stdout_[1]);
		::close(fds_stderr_[1]);
	}
}

pipe_prg::~pipe_prg()
{
	if (fds_stdout_[0] >= 0) ::close(fds_stdout_[0]);
	if (fds_stdout_[0] >= 0) ::close(fds_stderr_[0]);
}

//child subprocess
void pipe_prg::child(const std::string& execname, char *args[])
{
	::close(fds_stdout_[0]);
	::close(fds_stderr_[0]);

	GLIBC_ASSERT(::dup2(fds_stdout_[1], 1));
	GLIBC_ASSERT(::dup2(fds_stderr_[1], 2));

	run(execname, args);
}

int pipe_prg::out_fd() const
{
	return fds_stdout_[0];
}

int pipe_prg::err_fd() const
{
	return fds_stderr_[0];
}

int pipe_prg::rc() const
{
	return child_rc_;
}

void pipe_prg::wait()
{
	int	status;

	if (::waitpid (child_pid_, &status, 0) != child_pid_)
		throw std::runtime_error("unable to waitpid");

	if (WIFEXITED (status))
		child_rc_ = WEXITSTATUS (status);
	else if (WIFSIGNALED (status))
		child_rc_ = 128 + WTERMSIG (status);
}

void pipe_prg::kill(int sig)
{
	if (child_pid_ > 0) ::kill(child_pid_, sig); //send signal to child
}

