/* pipeprg.cc
* This file is part of Colorifer
* Copyright (c) 2002,2003,2004 by Stanislav Ievlev
*
* This file is covered by the GNU General Public License,
* which should be included with colorifer as the file COPYING.
*/

#include <stack>
#include <boost/bind.hpp>

//for debug
#include <iostream>

#include <subst.hh>

using namespace colorifer;

namespace
{
    /**
     * helper functor to call method with binding
     */
    void ptr_call(const pattern_ptr& pat,const std::string& line,color_map& map)
    {
	pat->fill(line,map);
    }
}    

void subst::process(const std::string& line, std::ostream& out)
{
	subcolors_.clear();

	//put default positions with the fake color
	subcolors_[0].push_back(std::make_pair(end_color, A_NORMAL));
	subcolors_[line.length()].push_back(std::make_pair(end_color, A_NORMAL));

	std::for_each(patterns_.begin(),
		      patterns_.end(),
		      boost::bind(ptr_call, _1, boost::cref(line), boost::ref(subcolors_)));
		

	const color_map::const_iterator end = subcolors_.end();
	std::stack<color_pair>	level;
	for (color_map::const_iterator it = subcolors_.begin();it != end;++it)
	{
		//compare with the latest pattern at this point
		if ((!level.empty()) && (end_color == it->second.rbegin()->first))
		{ //compare with default color, symbol of the 'end'
			//return to the previous color in the stack: leave default color always in stack
			if (level.size() > 1) level.pop();
		}
		else
		{
			//push to stack all candidates at this point
			std::for_each(it->second.begin(),
				      it->second.end(),
				      boost::bind(&std::stack<color_pair>::push, &level, _1));
		}

		out << ncurses::setfg(level.top().first, level.top().second);
		color_map::const_iterator next = it;
		out << line.substr(it->first, (++next)->first - it->first); //print substring of the string
	}
	out << std::endl;
}

void subst::add(const std::string& line)
{
    pattern *item = 0;
    if ('/' == line[0])
	item = new regex_pattern(line);
    else if ('s' == line[0])
	item = new regsubst_pattern(line);
    else if (::isdigit(line[0]))
	item = new lnum_pattern(line);
    else
	item = new addr_pattern(line);

    patterns_.push_back(pattern_ptr(item));
}

void subst::load(std::istream& is)
{
	std::string line;
	while (std::getline(is, line))
	{
		if (line.empty() || ('#' == line[0])) continue;
		add(line);
	}
}
